#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const pokemon_json_1 = __importDefault(require("../pokemon.json"));
const path_1 = __importDefault(require("path"));
const term_img_1 = __importDefault(require("term-img"));
const execa_1 = __importDefault(require("execa"));
const fuse_js_1 = __importDefault(require("fuse.js"));
const meow = require("meow");
const fuse = new fuse_js_1.default(Object.values(pokemon_json_1.default), {
    keys: ["name", "filename"],
    isCaseSensitive: false,
    shouldSort: true,
});
const cli = meow(`
	Usage
	  $ pokemonshow <nameOrNumber>

	Options
		--xterm, -x  Show xterm instead of image in iTerm
    --say, -s Announces the name of the Pokémon
    --list Lists all the available Pokémon

  Examples
    $ pokemonshow
    $ pokemonshow pikachu
    $ pokemonshow 025 -x
    $ pokemonshow 42 -s
    `, {
    flags: {
        xterm: {
            type: "boolean",
            alias: "x",
        },
        say: {
            type: "boolean",
            alias: "s",
        },
        list: {
            type: "boolean",
        },
    },
});
function getPokemonFromInput(nameOrNumber) {
    if (!nameOrNumber) {
        const names = Object.keys(pokemon_json_1.default).filter((i) => !Number.isInteger(Number(i)));
        const randomName = names[Math.floor(Math.random() * names.length)];
        return pokemon_json_1.default[randomName];
    }
    // Is the input a number? If so then let's use it.
    if (Number.isInteger(Number(nameOrNumber)) && Number(nameOrNumber) > 0) {
        return pokemon_json_1.default[nameOrNumber];
    }
    else {
        // Try finding the given pokemon using its "alias"
        const pokemonUsingKey = pokemon_json_1.default[nameOrNumber.toLowerCase()];
        if (pokemonUsingKey) {
            return pokemonUsingKey;
        }
        // Otherwise, use fuzzy search to find a match.
        const fuzzyResults = fuse.search(nameOrNumber);
        if (!fuzzyResults || fuzzyResults.length < 1) {
            return undefined;
        }
        return fuzzyResults[0].item;
    }
}
async function displayImage(pokemon, useFallback, showName) {
    if (showName) {
        console.log(`It's ${pokemon.name}!\n`);
    }
    const fallback = async () => {
        const { stdout } = await execa_1.default("cat", [
            path_1.default.resolve(__dirname, "../xterms", pokemon.filename.replace(".png", "")),
        ]);
        console.log(stdout);
    };
    if (useFallback) {
        return fallback();
    }
    term_img_1.default(path_1.default.resolve(__dirname, "../images", pokemon.filename), {
        fallback,
    });
}
(async () => {
    if (cli.flags.list) {
        Object.entries(pokemon_json_1.default)
            .filter(([key]) => {
            if (Number.isInteger(Number(key))) {
                return false;
            }
            return true;
        })
            .forEach(([, value]) => {
            const numberMatch = value.filename.match(/^([0-9]{3})/);
            console.log({
                name: value.name,
                number: numberMatch ? numberMatch[0] : "",
                filename: value.filename,
            });
        });
        return;
    }
    const name = cli.input[0];
    const pokemon = getPokemonFromInput(name);
    if (!pokemon) {
        return;
    }
    displayImage(pokemon, !!cli.flags.xterm, !!cli.flags.say);
})();
